@@ -12,11 +12,11 @@ from typing import Any, Dict, List, Optional, Sequence
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action        # type: ignore
 
 
-__version__ = '0.0.2'
+__version__ = '0.0.3'
 
 
 FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper
-FLOW_PATH_TEXT_SEP = " / "           #: flow path separator used by :func:`flow_path_text`
+FLOW_PATH_TEXT_SEP = " / "           #: flow path separator used by :meth:`~LiszDataMixin.flow_path_text`
 
 HELP_TEXT = (
     "Touch item name for to toggle selection.\n"
@@ -35,7 +35,7 @@ HELP_TEXT = (
 )
 
 
-LiszItem = Dict[str, Any]               #: item data (lid) type
+LiszItem = Dict[str, Any]               #: node item data (nid) type
 LiszNode = List[LiszItem]               #: node/list type
 
 
@@ -52,7 +52,10 @@ def check_item_id(item_id: str) -> bool:
 class LiszDataMixin(ABC):
     """ lisz data model - independent from used GUI framework. """
     root_node: LiszNode             #: root of lisz data structure
-    current_node: LiszNode          #: currently selected/displayed node / sub list
+    current_node_items: LiszNode    #: node item data of the current node / sub list (stored as app state via root_node)
+    filtered_indexes: List[int]     #: indexes of the filtered/displayed items in the current node
+    filter_selected: bool           #: True for to hide/filter selected node items
+    filter_unselected: bool         #: True for to hide/filter unselected node items
 
     # mixin shadow attributes - implemented by :class:`~ae.console_app.ConsoleApp` or :class:`~ae.gui_app.MainAppBase`
     debug_level: int                #: :attr:`~AppBase.debug_level`
@@ -94,11 +97,11 @@ class LiszDataMixin(ABC):
         :param item_id:         item id to identify the item/sub-node to be deleted.
         :param node_only:       True if only delete the sub-node of the identified item, False for to delete the item.
         """
-        lid = self.item_by_id(item_id)
+        nid = self.item_by_id(item_id)
         if node_only:
-            lid.pop('node')
+            nid.pop('node')
         else:
-            self.current_node.remove(lid)
+            self.current_node_items.remove(nid)
             self.change_app_state('flow_id', id_of_flow('', ''), send_event=False)
 
         self.play_sound('deleted')
@@ -128,21 +131,21 @@ class LiszDataMixin(ABC):
             return f"item id {new_id} exists already"
 
         if add_item:                        # add new item
-            lid = dict(id=new_id)
+            nid = dict(id=new_id)
             if want_node:
-                lid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
-            self.current_node.insert(0, lid)
+                nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
+            self.current_node_items.insert(0, nid)
 
         else:                               # edit item
-            lid = self.current_node[old_idx]
-            lid['id'] = new_id
-            if want_node != ('node' in lid):    # NOTE: != operator has lower priority than in operator
+            nid = self.current_node_items[old_idx]
+            nid['id'] = new_id
+            if want_node != ('node' in nid):    # NOTE: != operator has lower priority than in operator
                 if want_node:
-                    lid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
-                elif lid['node']:       # let user confirm node deletion of non-empty lid['node']
+                    nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
+                elif nid['node']:       # let user confirm node deletion of non-empty nid['node']
                     return 'request_delete_confirmation_for_node'
                 else:
-                    lid.pop('node')     # remove empty node
+                    nid.pop('node')     # remove empty node
 
         self.play_sound('added' if add_item else 'edited')
 
@@ -156,9 +159,9 @@ class LiszDataMixin(ABC):
         :return:                item list index in the searched node or -1 if item id was not found.
         """
         if searched_node is None:
-            searched_node = self.current_node
-        for list_idx, lid in enumerate(searched_node):
-            if lid['id'] == item_id:
+            searched_node = self.current_node_items
+        for list_idx, nid in enumerate(searched_node):
+            if nid['id'] == item_id:
                 return list_idx
         return -1
 
@@ -177,10 +180,10 @@ class LiszDataMixin(ABC):
             key = flow_key(flow_id)
             key_len = len(key)
             id_len = 6 if landscape else 3
-            for lid in self.current_node:
-                lid_id = lid['id']
-                if lid_id != key:
-                    while lid_id.startswith(key[:id_len]) and id_len < key_len:
+            for nid in self.current_node_items:
+                item_id = nid['id']
+                if item_id != key:
+                    while item_id.startswith(key[:id_len]) and id_len < key_len:
                         id_len += 1
             return f" ->{key[:id_len]}"
         return f".{flow_id}" if self.debug_level else ""
@@ -191,18 +194,18 @@ class LiszDataMixin(ABC):
         :param text:    flow path text - like returned by :meth:`~LiszDataMixin.flow_path_text`.
         :return:        flow path list.
         """
-        path_lid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
+        path_nid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
         flow_path = list()
         if text != FLOW_PATH_ROOT_ID:
             node = self.root_node
             for part in text.split(FLOW_PATH_TEXT_SEP):
-                for lid in node:
-                    if lid['id'].startswith(part) and 'node' in lid:
-                        part = lid['id']
-                        path_lid = lid
+                for nid in node:
+                    if nid['id'].startswith(part) and 'node' in nid:
+                        part = nid['id']
+                        path_nid = nid
                         break
-                flow_path.append(id_of_flow('enter', 'node', part))
-                node = path_lid['node']         # type: ignore   # mypy not supports recursive types see issue #731
+                flow_path.append(id_of_flow('enter', 'item', part))
+                node = path_nid['node']         # type: ignore   # mypy not supports recursive types see issue #731
         return flow_path
 
     def flow_path_text(self, flow_path: List[str], min_len: int = 3) -> str:
@@ -212,7 +215,7 @@ class LiszDataMixin(ABC):
         :param flow_path:
         :return:
         """
-        path_lid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
+        path_nid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
         shortened_ids = list()
         node = self.root_node
         for flow_id in flow_path:
@@ -221,16 +224,16 @@ class LiszDataMixin(ABC):
             id_len = min_len
             node_id = flow_key(flow_id)
             sub_id_len = len(node_id)
-            for lid in node:
-                if lid['id'] == node_id:
-                    assert 'node' in lid
-                    path_lid = lid
-                elif 'node' in lid:
-                    while lid['id'].startswith(node_id[:id_len]) and id_len < sub_id_len:
+            for nid in node:
+                if nid['id'] == node_id:
+                    assert 'node' in nid
+                    path_nid = nid
+                elif 'node' in nid:
+                    while nid['id'].startswith(node_id[:id_len]) and id_len < sub_id_len:
                         id_len += 1
 
             shortened_ids.append(node_id[:id_len])
-            node = path_lid['node']         # type: ignore   # mypy not supports recursive types see issue #731
+            node = path_nid['node']         # type: ignore   # mypy not supports recursive types see issue #731
 
         return FLOW_PATH_TEXT_SEP.join(shortened_ids)
 
@@ -242,23 +245,23 @@ class LiszDataMixin(ABC):
         """
         if flow_path is None:
             flow_path = self.flow_path
-        current_node = self.root_node
+        current_node_items = self.root_node
         for flow_id in flow_path:
             if flow_action(flow_id) != 'enter':
                 break
             node_id = flow_key(flow_id)
-            current_node = self.item_by_id(node_id, searched_node=current_node)['node']
-        return current_node
+            current_node_items = self.item_by_id(node_id, searched_node=current_node_items)['node']
+        return current_node_items
 
     def flow_path_quick_jump_nodes(self):
         """ determine nodes relative to the current flow path to quick-jump to from current node. """
         def append_nodes(node):
             """ recursively collect all available nodes (possible flow paths) """
-            for lid in node:
-                if 'node' in lid:
-                    deeper_flow_path.append(id_of_flow('enter', 'item', lid['id']))
+            for nid in node:
+                if 'node' in nid:
+                    deeper_flow_path.append(id_of_flow('enter', 'item', nid['id']))
                     paths.append(self.flow_path_text(deeper_flow_path))
-                    append_nodes(lid['node'])
+                    append_nodes(nid['node'])
                     deeper_flow_path.pop()
 
         paths = [FLOW_PATH_ROOT_ID] if self.flow_path else list()
@@ -268,10 +271,27 @@ class LiszDataMixin(ABC):
             paths.append(self.flow_path_text(self.flow_path[:deep]))
 
         deeper_flow_path = self.flow_path[:deep]
-        append_nodes(self.current_node)
+        append_nodes(self.current_node_items)
 
         return paths
 
+    def focus_neighbour_item(self, delta: int):
+        """ move flow id to previous/next displayed/filtered node item.
+
+        :param delta:       moving step (if greater 0 then forward, else backward).
+        """
+        filtered_indexes = self.filtered_indexes
+        if filtered_indexes:
+            flow_id = self.flow_id
+            if flow_id:
+                item_idx = self.find_item_index(flow_key(flow_id))
+                assert item_idx >= 0
+                filtered_idx = filtered_indexes.index(item_idx)
+                idx = min(max(0, filtered_idx + delta), len(filtered_indexes) - 1)
+            else:
+                idx = min(max(-1, delta), 0)
+            self.change_flow(id_of_flow('focus', 'item', self.current_node_items[filtered_indexes[idx]]['id']))
+
     def item_by_id(self, item_id: str, searched_node: Optional[LiszNode] = None) -> LiszItem:
         """ search item in either the passed or the current node.
 
@@ -280,28 +300,28 @@ class LiszDataMixin(ABC):
         :return:                found item or empty dict with new/empty id if not found.
         """
         if searched_node is None:
-            searched_node = self.current_node
+            searched_node = self.current_node_items
         index = self.find_item_index(item_id, searched_node=searched_node)
         if index != -1:
             return searched_node[index]
         return dict(id='')
 
     def move_item(self, dragged_node: LiszNode, item_id: str, node_idx: int):
-        """ move item with id from passed dragged_node list to index node_idx in :attr:`~LiszDataMixin.current_node`.
+        """ move item from passed dragged_node list to index node_idx in :attr:`~LiszDataMixin.current_node_items`.
 
         :param dragged_node:    node where the item to moved from.
         :param item_id:         id of the moved item.
         :param node_idx:        index in the node where the item has to be moved to.
         """
         src_idx = self.find_item_index(item_id, searched_node=dragged_node)
-        lid = dragged_node.pop(src_idx)
-        if dragged_node == self.current_node and node_idx > src_idx:
+        nid = dragged_node.pop(src_idx)
+        if dragged_node == self.current_node_items and node_idx > src_idx:
             node_idx -= 1
-        self.current_node.insert(node_idx, lid)
+        self.current_node_items.insert(node_idx, nid)
 
     def on_app_init(self):
-        """ initialize self.current_node from flow_id/flow_path saved on last app stop. """
-        self.refresh_current_node_from_flow_path()
+        """ initialize self.current_node_items from flow_id/flow_path saved on last app stop. """
+        self.refresh_current_node_items_from_flow_path()
 
     def on_filter_toggle(self, flow_id: str, _event_kwargs: Dict[str, Any]) -> bool:
         """ toggle filter on click of either the selected or the unselected filter button.
@@ -339,17 +359,17 @@ class LiszDataMixin(ABC):
         # handle hot key without a modifier key and while in item list, first current item flow changes
         flo_key = flow_key(self.flow_id)
         if key_code == 'up':
-            self.set_neighbour_flow(-1)
+            self.focus_neighbour_item(-1)
         elif key_code == 'down':
-            self.set_neighbour_flow(1)
+            self.focus_neighbour_item(1)
         elif key_code == 'pgup':
-            self.set_neighbour_flow(-15)
+            self.focus_neighbour_item(-15)
         elif key_code == 'pgdown':
-            self.set_neighbour_flow(15)
+            self.focus_neighbour_item(15)
         elif key_code == 'home':
-            self.set_neighbour_flow(-999999)
+            self.focus_neighbour_item(-999999)
         elif key_code == 'end':
-            self.set_neighbour_flow(999999)
+            self.focus_neighbour_item(999999)
 
         # toggle selection of current item
         elif key_code == ' ' and self.flow_id:    # key string 'space' is not in Window.command_keys
@@ -358,9 +378,9 @@ class LiszDataMixin(ABC):
 
         # enter/leave flow in current list
         elif key_code in ('enter', 'right') and self.flow_id and 'node' in self.item_by_id(flo_key):
-            self.change_flow(id_of_flow('enter', 'node', flo_key))
+            self.change_flow(id_of_flow('enter', 'item', flo_key))
         elif key_code in ('escape', 'left') and self.flow_path:
-            self.change_flow(id_of_flow('leave', 'node'))
+            self.change_flow(id_of_flow('leave', 'item'))
 
         # item processing: add, edit or request confirmation of deletion of current item
         elif key_code in ('a', '+'):
@@ -384,7 +404,7 @@ class LiszDataMixin(ABC):
         :param _event_kwargs:   event kwargs.
         :return:                True for to process/change flow id.
         """
-        self.play_sound(id_of_flow('enter', 'node'))
+        self.play_sound(id_of_flow('enter', 'item'))
 
     def on_node_leave(self, _flow_id: str, _event_kwargs: dict):
         """ leaving sub node, setting current node to parent node.
@@ -393,7 +413,7 @@ class LiszDataMixin(ABC):
         :param _event_kwargs:   event kwargs.
         :return:                True for to process/change flow id.
         """
-        self.play_sound(id_of_flow('leave', 'node'))
+        self.play_sound(id_of_flow('leave', 'item'))
 
     def on_node_jump(self, flow_id: str, _event_kwargs: dict) -> bool:
         """ FlowButton clicked event handler restoring flow path from the flow key.
@@ -416,7 +436,7 @@ class LiszDataMixin(ABC):
         self.change_app_state('flow_path', flow_path)
         self.change_app_state('flow_id', id_of_flow('', ''))
 
-        self.play_sound(id_of_flow('enter', 'node'))
+        self.play_sound(id_of_flow('enter', 'item'))
         self.refresh_all()
         return True
 
@@ -427,7 +447,7 @@ class LiszDataMixin(ABC):
         if self.debug_level:
             self.play_sound('debug_draw')
 
-        self.refresh_current_node_from_flow_path()
+        self.refresh_current_node_items_from_flow_path()
 
         # save last actual flow id (because refreshed/redrawn widget observers could change flow id via focus)
         flow_id = self.flow_id
@@ -447,20 +467,9 @@ class LiszDataMixin(ABC):
         assert self._refreshing_data
         self._refreshing_data = False
 
-    def refresh_current_node_from_flow_path(self):
+    def refresh_current_node_items_from_flow_path(self):
         """ refresh current node including the depending display node. """
-        self.current_node = self.flow_path_node()
-
-    def set_neighbour_flow(self, delta):
-        """ move flow id to previous/next item. """
-        current_node = self.current_node
-        if current_node:
-            flow_id = self.flow_id
-            if flow_id:
-                idx = min(max(0, self.find_item_index(flow_key(flow_id)) + delta), len(current_node) - 1)
-            else:
-                idx = min(max(-1, delta), 0)
-            self.change_app_state('flow_id', id_of_flow('focus', 'item', current_node[idx]['id']))
+        self.current_node_items = self.flow_path_node()
 
     def sub_item_ids(self, item_id, node_only, node=None, sub_item_ids=None):
         """ return item names of all items including their sub-node items (if exists).
@@ -474,20 +483,19 @@ class LiszDataMixin(ABC):
         :return:                list of found item ids.
         """
         if node is None:
-            node = self.current_node
+            node = self.current_node_items
         if sub_item_ids is None:
             sub_item_ids = list()
 
         if not node_only:
             sub_item_ids.append(item_id)
 
-        lid = self.item_by_id(item_id, node)
-        node = lid.get('node', list())
-        for sub_lid in node:
-            if sub_lid.get('node'):
-                self.sub_item_ids(sub_lid['id'], False, node, sub_item_ids)
+        node = self.item_by_id(item_id, node).get('node', list())
+        for nid in node:
+            if nid.get('node'):
+                self.sub_item_ids(nid['id'], False, node, sub_item_ids)
             else:
-                sub_item_ids.append(sub_lid['id'])
+                sub_item_ids.append(nid['id'])
 
         return sub_item_ids
 
@@ -496,8 +504,8 @@ class LiszDataMixin(ABC):
 
         :param node_idx:        list index of the item in the current node to change the selection for.
         """
-        new_sel = not self.current_node[node_idx].get('sel', False)
+        new_sel = not self.current_node_items[node_idx].get('sel', False)
         if new_sel:
-            self.current_node[node_idx]['sel'] = 1
+            self.current_node_items[node_idx]['sel'] = 1
         else:
-            self.current_node[node_idx].pop('sel')
+            self.current_node_items[node_idx].pop('sel')
